/*****************************************************************************
    dllapps.h

    Diese Datei ist Teil des Programmpakets 'ACL-tftp'.
    Alle Rechte bei Andreas G. Schindler
                    Klappacher Str. 2i
                    64285 Darmstadt, Deutschland.

    Dieses Programm ist 'Freie Software'; Sie knnen es entsprechend den 
    Bedingungen der 'Allgemeinen ffentlichen GNU-Lizenz' (verffentlicht 
    durch die 'Free Software Foundation') weiter verbreiten und/oder 
    modifizieren. Massgeblich ist die Version 2 oder eine neuere Version
    dieser Lizenz. 

    Dieses Programm wird in der Hoffnung dass es ntzlich is weitergegeben,
    jedoch OHNE JEDE GARANTIE. Dieser Garantieauschluss bezieht sich auch
    auf Eigenschaften wie Marktfhigkeit oder Eignung zu einem bestimmten
    Zweck. Details finden sich in der 'Allgemeinen ffentlichen GNU-Lizenz'.

    Eine Kopie dieser Lizenz sollte in  Programmpaket, dem diese Datei
    entstammt, enthalten sein. Falls nicht, schreiben Sie an die 
    'Free Software Foundation, Inc.', 59 Temple Place, Suite 330, Boston, 
    MA  02111-1307  USA.


    This file is part of the 'ACL-tftp' Package
    Copyright (C) 1999-2003 by Andreas G. Schindler
                               Klappacher Str. 2i
                               Darmstadt, Germany.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************/
#ifndef DLLAPPS_H
#define DLLAPPS_H

#include "types.h"
#include "acl.h"
#ifdef HAVE_REGEX_H
#include <sys/types.h>
#include <regex.h>
#else
#include "regex.h"
#endif
#include "netio.h"
#include "dll.h"

#define	CTYPE_REGEX	2
#define	CTYPE_ACL	3
#define	CTYPE_PID	4
#define	CTYPE_FILTER	5
#define CTYPE_TIMES	6
#define CTYPE_BLIST	7
#define CTYPE_CACHE	8

#define FIND_FIRST	0
#define FIND_LAST	1
#define FIND_MIN	2
#define FIND_MAX	3

typedef struct dlr_s {	struct dlr_s	*next;
			struct dlr_s	*prev;
			int		ctype;
			int		mark;
			int		filter;
			int		dtype;
			u_long		maxsize;
			char		*map;
			regex_t		compiled;
			char		*raw;		} dlr_t;

typedef struct dla_s {	struct dla_s	*next;
			struct dla_s	*prev;
			int		ctype;
			int		mark;
			struct acl_s	acl;		} dla_t;

typedef struct dlp_s {	struct dlp_s	*next;
			struct dlp_s	*prev;
			int		ctype;
			int		mark;
			pid_t		pid;
			sai_t		peer;		} dlp_t;

typedef struct dlf_s {	struct dlf_s	*next;
			struct dlf_s	*prev;
			int		ctype;
			int		mark;
			char		*rxfilter;
			char		*txfilter;	} dlf_t;

typedef struct dlt_s {	struct dlt_s	*next;
			struct dlt_s	*prev;
			int		ctype;
			int		mark;
			tout_t		times;		} dlt_t;

typedef struct dlb_s {	struct dlb_s	*next;
			struct dlb_s	*prev;
			int		ctype;
			int		first;
			int		last;		} dlb_t;

typedef struct dlc_s {	struct dlc_s	*next;
			struct dlc_s	*prev;
			int		ctype;
			int		mark;
			char		*name;
			struct stat	status;
			char		*cache;
			ulong		size;
			int		flags;
			char*		sem;
			time_t		used;		} dlc_t;

#define filepat rxfilter
#define filemap txfilter

void clearReChain(dlr_t *head);
void clearPidChain(dlp_t *head);
void clearFilterChain(dlf_t *head);
void deleteReChain(dlr_t *head);
void deletePidChain(dlp_t *head);
void deleteACLChain(dla_t *head);
void deleteFilterChain(dlf_t *head);
void deleteBlistChain(dlb_t *head);
void deleteCacheChain(dlc_t *head);
dlr_t *initReChain(void);
dla_t *initACLChain(void);
dlp_t *initPidChain(void);
dlf_t *initFilterChain(void);
dlb_t *initBlistChain(void);
dlc_t *initCacheChain(void);
void deleteACLByMark(dla_t *head, int mark);
void deleteFilterByMark(dlf_t *head, int mark);
void deleteReByMark(dlr_t *head, int mark);
int deletePid(dlp_t *head, pid_t pid);
int deleteCacheEntry(dlc_t *head, dlc_t *entry);
dlr_t *findRePattern(dlr_t *head, dlr_t *start, int mark, char *name,int dtype);
dla_t *findACL(dla_t *head, dla_t *start, int mark, in_addr_t addr, int mode);
dlp_t *findPid(dlp_t *head, pid_t pid);
dlc_t *findCacheByMark(dlc_t *head, char *name, int mark);
dla_t *getACLByMark(dla_t *head, dla_t *start, int mark);
dlf_t *getFilterByMark(dlf_t *head, dlf_t *start, int mark);
dlr_t *getReByMark(dlr_t *head, dlr_t *start, int mark);
dlp_t *getPidByMark(dlp_t *head, dlp_t *start, int mark);
dlr_t *getNextRe(dlr_t *head, dlr_t *start);
dlc_t *getNextCache(dlc_t *head, dlc_t *start);
dla_t *getNextACL(dla_t *head, dla_t *start);
dlp_t *getNextPid(dlp_t *head, dlp_t *start);
dlb_t *getNextBlist(dlb_t *head, dlb_t *start);
int getPidCount(dlp_t *head);
pid_t *getPidVec(dlp_t *head);
int getPidCountByMark(dlp_t *head, int mark);
dlf_t *getNextFilter(dlf_t *head, dlf_t *start);
int addRePattern(dlr_t *head, int mark, char *pattern, char *map, int filter, int dtype, u_long maxsize, dlr_t **np);
int addACL(dla_t *head, int mark, int perm, char *addr, char *mask, 
           int reasolv, dla_t **np);
int addPid(dlp_t *head, int mark, pid_t pid, sai_t *peer, dlp_t **np);
int addBlist(dlb_t *head, int start, int end, dlb_t **np);
int addFilter(dlf_t *head, int mark, char *rxf, char *txf, dlf_t **np);
int addCache(dlc_t *head, int mark, char *name, struct stat *status, 
             int flags, dlc_t **np);
void sortFilterByMark(dlf_t *head, int mode);
void dumpBody (FILE *fd, dll_t *dp, int flag, int element);
void deleteTimerChain (dlt_t *head);
dlt_t *initTimerChain(void);
dlt_t *getNextTimer (dlt_t *head, dlt_t *start);
dlt_t *getTimerByMark (dlt_t *head, dlt_t *start, int mark);
int addTimer (dlt_t *head, int mark, tout_t *tp, dlt_t **np);
void sortTimerByMark (dlt_t *head, int mode);
void sortACLbyIP (dla_t *head);
int findNextInBlist (dlb_t *db, int val);
void dumpBody (FILE *fd, dll_t *dp, int flag, int element);

#endif /* DLLAPPS_H */
